Package: TPP2D
Title: Detection of ligand-protein interactions from 2D thermal
        profiles (DLPTP)
Version: 1.28.0
Authors@R: c(
    person("Nils", "Kurzawa", email = "nilskurzawa@gmail.com", 
        role = c("aut", "cre")),
    person("Holger", "Franken", role = c("aut")),
    person("Simon", "Anders", role = c("aut")),
    person("Wolfgang", "Huber", role = c("aut")),
    person("Mikhail M.", "Savitski", role = c("aut")))
Description: Detection of ligand-protein interactions from 2D thermal 
    profiles (DLPTP), Performs an FDR-controlled analysis of 2D-TPP 
    experiments by functional analysis of dose-response curves across 
    temperatures.
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
LazyData: false
biocViews: Software, Proteomics, DataImport
BugReports: https://support.bioconductor.org/
URL: http://bioconductor.org/packages/TPP2D
RoxygenNote: 7.3.3
Depends: R (>= 3.6.0), stats, utils, dplyr, methods
Imports: ggplot2, tidyr, foreach, doParallel, openxlsx, stringr, RCurl,
        parallel, MASS, BiocParallel, limma
Suggests: knitr, testthat, rmarkdown, BiocStyle
git_url: https://git.bioconductor.org/packages/TPP2D
git_branch: RELEASE_3_23
git_last_commit: 42f758f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:46:44 UTC; biocbuild
Author: Nils Kurzawa [aut, cre],
  Holger Franken [aut],
  Simon Anders [aut],
  Wolfgang Huber [aut],
  Mikhail M. Savitski [aut]
Maintainer: Nils Kurzawa <nilskurzawa@gmail.com>
Built: R 4.6.0; ; 2026-04-29 05:04:11 UTC; unix
