# Generated by roxygen2: do not edit by hand

export(TPP_importCheckConfigTable)
export(annotateDataList)
export(bootstrapNull)
export(bootstrapNullAlternativeModel)
export(bootstrapNullAlternativeModelFast)
export(competeModels)
export(computeFStatFromParams)
export(computeFstat)
export(configWide2Long)
export(filterOutContaminants)
export(findHits)
export(fitAndEvalDataset)
export(fitH0Model)
export(fitH1Model)
export(getFDR)
export(getModelParamsDf)
export(getPEC504Temperature)
export(getPvalues)
export(gg_qq)
export(import2dDataset)
export(import2dMain)
export(plot2dTppFcHeatmap)
export(plot2dTppFit)
export(plot2dTppProfile)
export(plot2dTppRelProfile)
export(plot2dTppVolcano)
export(recomputeSignalFromRatios)
export(renameColumns)
export(resolveAmbiguousProteinNames)
export(runTPP2D)
export(tpcaResult)
exportClasses(tpp2dExperiment)
import(BiocParallel)
import(dplyr)
import(ggplot2)
importFrom(MASS,lda)
importFrom(RCurl,url.exists)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(limma,squeezeVar)
importFrom(methods,is)
importFrom(methods,new)
importFrom(openxlsx,read.xlsx)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,approx)
importFrom(stats,coef)
importFrom(stats,density)
importFrom(stats,fft)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stringr,str_to_title)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(utils,head)
importFrom(utils,read.delim)
importFrom(utils,read.table)
