#' TPP2D defunct functions
#' 
#' @title Defunct functions in package \pkg{TPP2D}.
#' @description The functions listed below are defunct and will be removed in
#'   the near future. When possible, alternative functions with similar
#'   functionality are also mentioned. Help pages for deprecated functions are
#'   available at \code{help("-deprecated")}.
#' @name TPP2D-defunct
#' @keywords internal
NULL