% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppProfile}
\alias{plot2dTppProfile}
\title{Plot 2D thermal profile intensities of a protein 
of choice}
\usage{
plot2dTppProfile(df, name)
}
\arguments{
\item{df}{tidy data frame of a 2D-TPP dataset}

\item{name}{gene name (clustername) of protein that 
should be visualized}
}
\value{
A ggplot displaying the thermal profile of
a protein of choice in a datset of choice
}
\description{
Plot 2D thermal profile intensities of a protein 
of choice
}
\examples{

data("simulated_cell_extract_df")
plot2dTppProfile(simulated_cell_extract_df, "protein1")

}
